@@ -1,9 +1,9 @@
-use std::process::Command;
-
 use clap::Shell;
 use colored::Colorize;
 use indicatif::{ProgressBar, ProgressStyle};
+use man::prelude::*;
 use prettytable::{cell, row, Cell, Row, Table};
+use std::process::Command;
 use tempfile::Builder;
 use term_size;
 
@@ -65,6 +65,114 @@ fn main() {
         return;
     }
 
+    // if manpage is requested, print the manpage and exit
+    if matches.is_present("manpage") {
+        let page = Manual::new("decoreco")
+        .about("re-encode video and audio files to save space")
+        .author(Author::new("Justyn Boyer (Jabster28)").email("justynboyer@gmail.com"))
+        // also add a dry run flag
+        .flag(
+            Flag::new()
+                .short("d")
+                .long("dry-run")
+                .help("don't actually do anything"),
+        )
+        // and a depth option
+        .option(
+            Opt::new("depth")
+                .short("D")
+                .long("depth")
+                .help("set the depth of the tree to search for files"),
+        )
+        // and a list flag
+        .flag(
+            Flag::new()
+                .short("l")
+                .long("list")
+                .help("list all files that would be processed and their sizes"),
+        )
+        // and a sort flag
+        .flag(
+            Flag::new()
+                .short("s")
+                .long("sort")
+                .help("sort the list of files by size"),
+        )
+        // and a reverse flag
+        .flag(
+            Flag::new()
+                .short("r")
+                .long("reverse")
+                .help("reverse the sort order"),
+        )
+        // and a set option
+        .option(
+            Opt::new("set")
+                .short("S")
+                .long("set")
+                .help("process only these files, ignore path"),
+        )
+        // and a video codec option
+        .option(
+            Opt::new("video-codec")
+                .short("v")
+                .long("video-codec")
+                .help("set the video codec to use. see CODECS for more info")
+                .default_value("h264"),
+        )
+        // and an audio codec option
+        .option(
+            Opt::new("audio-codec")
+                .short("a")
+                .long("audio-codec")
+                .help("set the audio codec to use. see CODECS for more info")
+                .default_value("aac"),
+        )
+        .example(
+            Example::new()
+                .text("re-encode all video files in your downloads folder to h264 and aac")
+                .command("decoreco ~/Downloads"),
+        )
+        .example(
+            Example::new()
+                .text("re-encode all video files in your downloads folder to hevc and mp3")
+                .command("decoreco -v hevc -a mp3 ~/Downloads"),
+        )
+        .example(
+            Example::new()
+                .text("list all video files in your home folder and sort them by size")
+                .command("decoreco -l -s ~/"),
+        )
+        .example(
+            Example::new()
+                .text("perform a dry run of converting your movies folder to avi")
+                .command("decoreco -d -v avi ~/Movies"),
+        )
+                .custom(
+            Section::new("codecs")
+                .paragraph("the following codecs are supported in order of general size while retaining quality, smallest to largest:")
+                .paragraph("(video) hevc, vp9, [h264], , vp8").paragraph("(audio) [aac], opus, vorbis, mp3")
+                .paragraph("HEVC (also known as H.265) isn't supported by many web browsers or operating systems at the moment, and as such some videos might not play after you re-encode them. This codec should only be used if you don't plan on sharing the files over the internet without transcoding them (like using a media server such as plex or emby), or unless you're confident that your software and hardware can play it.").paragraph("Encoding HEVC also takes quite a bit longer thn h264, due to the higher compression ratio.")
+        );
+        // save to a tempdir
+        let tempdir = Builder::new().prefix("decoreco").tempdir().unwrap();
+        let manpage = tempdir.path().join("decoreco.1");
+        let mut file = std::fs::File::create(&manpage).unwrap();
+        std::io::Write::write_all(&mut file, page.render().as_bytes())
+            .and_then(|_| {
+                Command::new("man").arg(manpage).status().map(|status| {
+                    if status.success() {
+                        Ok(())
+                    } else {
+                        Err(std::io::Error::new(std::io::ErrorKind::Other, "man failed"))
+                    }
+                })
+            })
+            .unwrap()
+            .unwrap();
+        return;
+    }
+
     // run the appropriate command
     // don't search for files if set is specified
     let mut files: Vec<&str>;
@@ -193,9 +301,12 @@ fn main() {
             .arg("-i")
             .arg(file)
             .arg("-c:v")
-            .arg("libx264")
+            .arg(matches.value_of("codec").unwrap())
             .arg("-c:a")
-            .arg("aac")
+            .arg(matches.value_of("audio").unwrap())
+            // keep subs
+            .arg("-c:s")
+            .arg("copy")
             // keep metadata
             .arg("-map_metadata")
             .arg("0")
